@@ -28,10 +28,14 @@
 #include <config.h>
 #include "nautilus-application.h"
 
+
 #include "file-manager/fm-desktop-icon-view.h"
 #include "file-manager/fm-icon-view.h"
 #include "file-manager/fm-list-view.h"
 #include "file-manager/fm-search-list-view.h"
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include "nautilus-desktop-window.h"
 #include "nautilus-first-time-druid.h"
 #include "nautilus-main.h"
@@ -39,17 +43,19 @@
 #include "nautilus-shell.h"
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-object.h>
-#include <dirent.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string-list.h>
 #include <eel/eel-string.h>
 #include <eel/eel-vfs-extensions.h>
 #include <eel/eel-gtk-extensions.h>
+#include <gdk/gdkx.h>
+#include <gtk/gtkinvisible.h>
 #include <gtk/gtksignal.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
+#include <libgnomeui/gnome-authentication-manager.h>
 #include <libgnomeui/gnome-client.h>
 #include <libgnomeui/gnome-messagebox.h>
 #include <libgnomeui/gnome-stock-icons.h>
@@ -64,7 +70,8 @@
 #include <libnautilus-private/nautilus-bonobo-extensions.h>
 #include <libnautilus-private/nautilus-undo-manager.h>
 #include <libnautilus-private/nautilus-volume-monitor.h>
-#include <libnautilus-private/nautilus-authn-manager.h>
+#include <libnautilus-private/nautilus-desktop-link-monitor.h>
+#include <libnautilus-private/nautilus-directory-private.h>
 #include <bonobo-activation/bonobo-activation.h>
 
 /* Needed for the is_kdesktop_present check */
@@ -75,8 +82,8 @@
 #define SEARCH_LIST_VIEW_IID "OAFIID:Nautilus_File_Manager_Search_List_View"
 #define SHELL_IID	     "OAFIID:Nautilus_Shell"
 
-/* Keeps track of the one and only desktop window. */
-static NautilusDesktopWindow *nautilus_application_desktop_window;
+/* Keeps track of all the desktop windows. */
+static GList *nautilus_application_desktop_windows;
 
 /* Keeps track of all the nautilus windows. */
 static GList *nautilus_application_window_list;
@@ -84,9 +91,6 @@ static GList *nautilus_application_window_list;
 static gboolean need_to_show_first_time_druid     (void);
 static void     desktop_changed_callback          (gpointer                  user_data);
 static void     desktop_location_changed_callback (gpointer                  user_data);
-static void     volume_mounted_callback           (NautilusVolumeMonitor    *monitor,
-						   NautilusVolume           *volume,
-						   NautilusApplication      *application);
 static void     volume_unmounted_callback         (NautilusVolumeMonitor    *monitor,
 						   NautilusVolume           *volume,
 						   NautilusApplication      *application);
@@ -142,9 +146,9 @@ nautilus_application_instance_init (NautilusApplication *application)
 	/* Create an undo manager */
 	application->undo_manager = nautilus_undo_manager_new ();
 
-	/* Watch for volume mounts so we can restore open windows */
-	g_signal_connect_object (nautilus_volume_monitor_get (), "volume_mounted",
-				 G_CALLBACK (volume_mounted_callback), application, 0);
+	/* Watch for volume mounts so we can restore open windows
+	 * This used to be for showing new window on mount, but is not
+	 * used anymore */
 
 	/* Watch for volume unmounts so we can close open windows */
 	g_signal_connect_object (nautilus_volume_monitor_get (), "volume_unmounted",
@@ -278,56 +282,43 @@ nautilus_make_uri_list_from_shell_strv (const char * const *strv)
 static void
 migrate_old_nautilus_files (void)
 {
-	char *new_desktop_dir, *np;
-	char *old_desktop_dir, *op;
-	char *old_desktop_dir_new_name;
-	struct stat buf;
-	DIR *dir;
-	struct dirent *de;
+	char *new_desktop_dir;
+	char *old_desktop_dir;
+	char *migrated_file;
+	char *link_name;
+	char *link_path;
+	int fd;
 	
-	old_desktop_dir = g_strconcat (g_get_home_dir (), "/.nautilus/desktop", NULL);
-	if (stat (old_desktop_dir, &buf) == -1) {
+	old_desktop_dir = nautilus_get_gmc_desktop_directory ();
+	if (!g_file_test (old_desktop_dir, G_FILE_TEST_IS_DIR)) {
 		g_free (old_desktop_dir);
 		return;
 	}
-	if (!S_ISLNK (buf.st_mode)){
-		dir = opendir (old_desktop_dir);
-		if (dir == NULL) {
-			g_free (old_desktop_dir);
-			return;
-		}
-	
+	migrated_file = g_build_filename (old_desktop_dir, ".migrated", NULL);
+	if (!g_file_test (migrated_file, G_FILE_TEST_EXISTS)) {
+		link_name = g_filename_from_utf8 (_("Link To Old Desktop"), -1, NULL, NULL, NULL);
 		new_desktop_dir = nautilus_get_desktop_directory ();
+		link_path = g_build_filename (new_desktop_dir, link_name, NULL);
+	
 		
-		while ((de = readdir (dir)) != NULL){
-			if (de->d_name [0] == '.'){
-				if (de->d_name [0] == 0)
-					continue;
-				
-				if (de->d_name [1] == '.' && de->d_name [2] == 0)
-					continue;
-			}
-	
-			op = g_strconcat (old_desktop_dir, "/", de->d_name, NULL);
-			np = g_strconcat (new_desktop_dir, "/", de->d_name, NULL);
-	
-			rename (op, np);
-	
-			g_free (op);
-			g_free (np);
-		}
-
-		closedir (dir);
-
+		symlink ("../.gnome-desktop", link_path);
+		
+		g_free (link_name);
 		g_free (new_desktop_dir);
+		g_free (link_path);
+
+		fd = creat (migrated_file, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
+		if (fd >= 0) {
+			close (fd);
+		}
+		
+		eel_show_info_dialog (_("The location of the desktop directory has changed in GNOME 2.4. "
+					"A link called \"Link To Old Desktop\" has been created on the desktop. "
+					"You can open this to move over the files you want, then delete the link."),
+				      _("Migrated old desktop"),
+				      NULL);
 	}
-
-	/* In case we miss something */
-	old_desktop_dir_new_name = g_strconcat (old_desktop_dir, "-old", NULL);
-	rename (old_desktop_dir, old_desktop_dir_new_name);
-	g_free (old_desktop_dir_new_name);
-
-	g_free (old_desktop_dir);
+	g_free (migrated_file);
 }
 
 static gint
@@ -367,10 +358,60 @@ finish_startup (NautilusApplication *application)
 	nautilus_sound_init ();
 
 	/* initialize URI authentication manager */
-	nautilus_authentication_manager_init ();
+	gnome_authentication_manager_init ();
 
 	/* Make the desktop work with old Nautilus. */
 	migrate_old_nautilus_files ();
+
+	/* Initialize the desktop link monitor singleton */
+	nautilus_desktop_link_monitor_get ();
+}
+
+static void
+initialize_kde_trash_hack (void)
+{
+	char *trash_dir;
+	char *desktop_dir, *desktop_uri, *kde_trash_dir;
+	char *dir, *basename;
+	char *kde_conf_file;
+	char *key;
+	gboolean def;
+	
+	trash_dir = NULL;
+
+	desktop_uri = nautilus_get_desktop_directory_uri_no_create ();
+	desktop_dir = gnome_vfs_get_local_path_from_uri (desktop_uri);
+	g_free (desktop_uri);
+	
+	if (g_file_test (desktop_dir, G_FILE_TEST_EXISTS)) {
+		/* Look for trash directory */
+		kde_conf_file = g_build_filename (g_get_home_dir(), ".kde/share/config/kdeglobals", NULL);
+		key = g_strconcat ("=", kde_conf_file, "=/Paths/Trash", NULL);
+		kde_trash_dir = gnome_config_get_string_with_default (key, &def);
+		gnome_config_drop_file (kde_conf_file);
+		g_free (kde_conf_file);
+		g_free (key);
+
+		if (kde_trash_dir != NULL) {
+			basename = g_path_get_basename (kde_trash_dir);
+			g_free (kde_trash_dir);
+			
+			dir = g_build_filename (desktop_dir, basename, NULL);
+
+			if (g_file_test (dir, G_FILE_TEST_IS_DIR)) {
+				trash_dir = g_strdup (basename);
+			} 
+			g_free (basename);
+			g_free (dir);
+		} 
+
+		if (trash_dir != NULL) {
+			nautilus_set_kde_trash_name (trash_dir);
+		}
+
+		g_free (trash_dir);
+	}
+	g_free (desktop_dir);
 }
 
 void
@@ -411,6 +452,8 @@ nautilus_application_startup (NautilusApplication *application,
 		nautilus_set_first_time_file_flag ();
 	}
 
+	initialize_kde_trash_hack ();
+
 	CORBA_exception_init (&ev);
 
 	/* Start up the factory. */
@@ -575,12 +618,84 @@ nautilus_application_startup (NautilusApplication *application,
 	CORBA_exception_free (&ev);
 }
 
+
+static void 
+selection_get_cb (GtkWidget          *widget,
+		  GtkSelectionData   *selection_data,
+		  guint               info,
+		  guint               time)
+{
+	/* No extra targets atm */
+}
+
+static GtkWidget *
+get_desktop_manager_selection (GdkDisplay *display, int screen)
+{
+	char *selection_name;
+	GdkAtom selection_atom;
+	Window selection_owner;
+	GtkWidget *selection_widget;
+
+	selection_name = g_strdup_printf ("_NET_DESKTOP_MANAGER_S%d", screen);
+	selection_atom = gdk_atom_intern (selection_name, FALSE);
+	g_free (selection_name);
+
+	selection_owner = XGetSelectionOwner (GDK_DISPLAY_XDISPLAY (display),
+					      gdk_x11_atom_to_xatom_for_display (display, 
+										 selection_atom));
+	if (selection_owner != None) {
+		return NULL;
+	}
+	
+	selection_widget = gtk_invisible_new_for_screen (gdk_display_get_screen (display, screen));
+	/* We need this for gdk_x11_get_server_time() */
+	gtk_widget_add_events (selection_widget, GDK_PROPERTY_CHANGE_MASK);
+
+	if (gtk_selection_owner_set_for_display (display,
+						 selection_widget,
+						 selection_atom,
+						 gdk_x11_get_server_time (selection_widget->window))) {
+		
+		g_signal_connect (selection_widget, "selection_get",
+				  G_CALLBACK (selection_get_cb), NULL);
+		return selection_widget;
+	}
+
+	gtk_widget_destroy (selection_widget);
+	
+	return NULL;
+}
+
 static void
-nautilus_application_create_desktop_window (NautilusApplication *application)
+desktop_unrealize_cb (GtkWidget        *widget,
+		      GtkWidget        *selection_widget)
+{
+	gtk_widget_destroy (selection_widget);
+}
+
+static gboolean
+selection_clear_event_cb (GtkWidget	        *widget,
+			  GdkEventSelection     *event,
+			  NautilusDesktopWindow *window)
+{
+	gtk_widget_destroy (GTK_WIDGET (window));
+	
+	nautilus_application_desktop_windows =
+		g_list_remove (nautilus_application_desktop_windows, window);
+
+	return TRUE;
+}
+
+static void
+nautilus_application_create_desktop_windows (NautilusApplication *application)
 {
 	static gboolean create_in_progress = FALSE;
+	GdkDisplay *display;
+	NautilusDesktopWindow *window;
+	GtkWidget *selection_widget;
+	int screens, i;
 
-	g_return_if_fail (nautilus_application_desktop_window == NULL);
+	g_return_if_fail (nautilus_application_desktop_windows == NULL);
 	g_return_if_fail (NAUTILUS_IS_APPLICATION (application));
 
 	if (create_in_progress) {
@@ -589,12 +704,32 @@ nautilus_application_create_desktop_window (NautilusApplication *application)
 
 	create_in_progress = TRUE;
 
-	nautilus_application_desktop_window = nautilus_desktop_window_new (application);
-	/* We realize it immediately so that the NAUTILUS_DESKTOP_WINDOW_ID
-	   property is set so gnome-settings-daemon doesn't try to set the
-	   background. And we do a gdk_flush() to be sure X gets it. */
-	gtk_widget_realize (GTK_WIDGET (nautilus_application_desktop_window));
-	gdk_flush ();
+	display = gdk_display_get_default ();
+	screens = gdk_display_get_n_screens (display);
+
+	for (i = 0; i < screens; i++) {
+		selection_widget = get_desktop_manager_selection (display, i);
+		if (selection_widget != NULL) {
+			window = nautilus_desktop_window_new (application,
+							      gdk_display_get_screen (display, i));
+			
+			g_signal_connect (selection_widget, "selection_clear_event",
+					  G_CALLBACK (selection_clear_event_cb), window);
+			
+			g_signal_connect (window, "unrealize",
+					  G_CALLBACK (desktop_unrealize_cb), selection_widget);
+			
+			/* We realize it immediately so that the NAUTILUS_DESKTOP_WINDOW_ID
+			   property is set so gnome-settings-daemon doesn't try to set the
+			   background. And we do a gdk_flush() to be sure X gets it. */
+			gtk_widget_realize (GTK_WIDGET (window));
+			gdk_flush ();
+
+			
+			nautilus_application_desktop_windows =
+				g_list_prepend (nautilus_application_desktop_windows, window);
+		}
+	}
 
 	create_in_progress = FALSE;
 }
@@ -602,17 +737,19 @@ nautilus_application_create_desktop_window (NautilusApplication *application)
 void
 nautilus_application_open_desktop (NautilusApplication *application)
 {
-	if (nautilus_application_desktop_window == NULL) {
-		nautilus_application_create_desktop_window (application);
+	if (nautilus_application_desktop_windows == NULL) {
+		nautilus_application_create_desktop_windows (application);
 	}
 }
 
 void
 nautilus_application_close_desktop (void)
 {
-	if (nautilus_application_desktop_window != NULL) {
-		gtk_widget_destroy (GTK_WIDGET (nautilus_application_desktop_window));
-		nautilus_application_desktop_window = NULL;
+	if (nautilus_application_desktop_windows != NULL) {
+		g_list_foreach (nautilus_application_desktop_windows,
+				(GFunc) gtk_widget_destroy, NULL);
+		g_list_free (nautilus_application_desktop_windows);
+		nautilus_application_desktop_windows = NULL;
 	}
 }
 
@@ -720,7 +857,8 @@ nautilus_window_unrealize_event_callback (GtkWidget *widget,
 }
 
 NautilusWindow *
-nautilus_application_create_window (NautilusApplication *application)
+nautilus_application_create_window (NautilusApplication *application,
+				    GdkScreen           *screen)
 {
 	NautilusWindow *window;
 
@@ -728,8 +866,10 @@ nautilus_application_create_window (NautilusApplication *application)
 	
 	window = NAUTILUS_WINDOW (gtk_widget_new (nautilus_window_get_type (),
 						  "app", application,
-						  "app_id", "nautilus", NULL));
-	
+						  "app_id", "nautilus",
+						  "screen", screen,
+						  NULL));
+
 	g_signal_connect (window, "delete_event",
 			  G_CALLBACK (nautilus_window_delete_event_callback), NULL);
 
@@ -756,9 +896,9 @@ nautilus_application_create_window (NautilusApplication *application)
 static void
 desktop_location_changed_callback (gpointer user_data)
 {
-	if (nautilus_application_desktop_window != NULL) {
-		nautilus_desktop_window_update_directory
-			(nautilus_application_desktop_window);
+	if (nautilus_application_desktop_windows != NULL) {
+		g_list_foreach (nautilus_application_desktop_windows,
+				(GFunc) nautilus_desktop_window_update_directory, NULL);
 	}
 }
 
@@ -826,27 +966,6 @@ need_to_show_first_time_druid (void)
 	return result;
 }
 
-static void
-volume_mounted_callback (NautilusVolumeMonitor *monitor, NautilusVolume *volume,
-			 NautilusApplication *application)
-{
-	NautilusWindow *window;
-	char *uri;
-
-	if (volume == NULL || application == NULL) {
-		return;
-	}
-
-	/* Open a window to the CD if the user has set that preference. */
-	if (nautilus_volume_get_device_type (volume) == NAUTILUS_DEVICE_CDROM_DRIVE
-		&& eel_gconf_get_boolean( "/apps/magicdev/do_fileman_window")) {
-		window = nautilus_application_create_window (application);
-		uri = gnome_vfs_get_uri_from_local_path (nautilus_volume_get_mount_path (volume));
-		nautilus_window_go_to (window, uri);
-		g_free (uri);
-	}
-}
-
 static gboolean
 window_can_be_closed (NautilusWindow *window)
 {
@@ -987,7 +1106,7 @@ update_session (gpointer callback_data)
 			      * if we have a desktop window. Prevents the
 			      * session thrashing that's seen otherwise
 			      */
-			     && nautilus_application_desktop_window != NULL);
+			     && nautilus_application_desktop_windows != NULL);
 }
 
 static void
